@@ -1,4 +1,4 @@
-/* $Id: conns.c,v 1.17.2.1 2004-08-06 16:56:55 rjkaes Exp $
+/* $Id$
  *
  * Create and free the connection structure. One day there could be
  * other connection related tasks put here, but for now the header
@@ -46,7 +46,7 @@ initialize_conn(int client_fd, const char* ipaddr, const char* string_addr)
 	/*
 	 * Allocate the space for the conn_s structure itself.
 	 */
-	connptr = safemalloc(sizeof(struct conn_s));
+	connptr = (struct conn_s*)safemalloc(sizeof(struct conn_s));
 	if (!connptr)
 		goto error_exit;
 
@@ -60,7 +60,6 @@ initialize_conn(int client_fd, const char* ipaddr, const char* string_addr)
 
 	/* These store any error strings */
 	connptr->error_variables = NULL;
-	connptr->error_variable_count = 0;
 	connptr->error_string = NULL;
 	connptr->error_number = -1;
 
@@ -75,10 +74,12 @@ initialize_conn(int client_fd, const char* ipaddr, const char* string_addr)
 	connptr->client_ip_addr = safestrdup(ipaddr);
 	connptr->client_string_addr = safestrdup(string_addr);
 
-	connptr->upstream_proxy = NULL;
-
 	update_stats(STAT_OPEN);
 
+#ifdef REVERSE_SUPPORT
+	connptr->reversepath = NULL;
+#endif
+
 	return connptr;
 
 error_exit:
@@ -115,17 +116,8 @@ destroy_conn(struct conn_s *connptr)
 	if (connptr->request_line)
 		safefree(connptr->request_line);
 
-	if (connptr->error_variables) {
-		int i;
-
-		for (i = 0; i != connptr->error_variable_count; ++i) {
-			safefree(connptr->error_variables[i]->error_key);
-			safefree(connptr->error_variables[i]->error_val);
-			safefree(connptr->error_variables[i]);
-		}
-
-		safefree(connptr->error_variables);
-	}
+	if (connptr->error_variables)
+		hashmap_delete(connptr->error_variables);
 
 	if (connptr->error_string)
 		safefree(connptr->error_string);
@@ -135,6 +127,11 @@ destroy_conn(struct conn_s *connptr)
 	if (connptr->client_string_addr)
 		safefree(connptr->client_string_addr);
 
+#ifdef REVERSE_SUPPORT
+	if (connptr->reversepath)
+		safefree(connptr->reversepath);
+#endif
+
 	safefree(connptr);
 
 	update_stats(STAT_CLOSE);
