@@ -1,4 +1,4 @@
-/* $Id: heap.h,v 1.3 2003-05-31 23:04:15 rjkaes Exp $
+/* $Id$
  *
  * See 'heap.c' for a detailed description.
  *
@@ -38,7 +38,7 @@ extern char *debugging_strdup(const char* s, const char* file,
 #  define saferealloc(x, y) debugging_realloc(x, y, __FILE__, __LINE__)
 #  define safestrdup(x) debugging_strdup(x, __FILE__, __LINE__)
 #  define safefree(x) do { \
-void **__safefree_tmp = (void *)&(x); \
+void **__safefree_tmp = (void**)(&(x)); \
 debugging_free(*__safefree_tmp, __FILE__, __LINE__); \
 *__safefree_tmp = NULL; \
 } while (0)
@@ -47,7 +47,7 @@ debugging_free(*__safefree_tmp, __FILE__, __LINE__); \
 #  define safemalloc(x) malloc(x)
 #  define saferealloc(x, y) realloc(x, y)
 #  define safefree(x) do { \
-void **__safefree_tmp = (void *)&(x); \
+void **__safefree_tmp = (void**)(&(x)); \
 free(*__safefree_tmp); \
 *__safefree_tmp = NULL; \
 } while (0)
