@@ -1,4 +1,4 @@
-/* $Id: reqs.c,v 1.105.2.2 2004-08-06 16:56:55 rjkaes Exp $
+/* $Id$
  *
  * This is where all the work in tinyproxy is actually done. Incoming
  * connections have a new child created for them. The child then
@@ -33,7 +33,6 @@
 #include "htmlerror.h"
 #include "log.h"
 #include "network.h"
-#include "regexp.h"
 #include "reqs.h"
 #include "sock.h"
 #include "stats.h"
@@ -117,7 +116,7 @@ add_connect_port_allowed(int port)
 static int
 check_allowed_connect_ports(int port)
 {
-	size_t i;
+	ssize_t i;
 	int *data;
 
 	/*
@@ -128,7 +127,7 @@ check_allowed_connect_ports(int port)
 		return 1;
 
 	for (i = 0; i != vector_length(ports_allowed_by_connect); ++i) {
-		data = vector_getentry(ports_allowed_by_connect, i, NULL);
+		data = (int*)vector_getentry(ports_allowed_by_connect, i, NULL);
 		if (!data)
 			return -1;
 
@@ -160,7 +159,7 @@ read_request_line(struct conn_s *connptr)
 	}
 
 	/*
-	 * Strip the new line and character return from the string.
+	 * Strip the new line and carriage return from the string.
 	 */
 	if (chomp(connptr->request_line, len) == len) {
 		/*
@@ -225,8 +224,8 @@ strip_username_password(char* host)
 static int
 extract_http_url(const char *url, struct request_s *request)
 {
-	request->host = safemalloc(strlen(url) + 1);
-	request->path = safemalloc(strlen(url) + 1);
+	request->host = (char*)safemalloc(strlen(url) + 1);
+	request->path = (char*)safemalloc(strlen(url) + 1);
 
 	if (!request->host || !request->path)
 		goto ERROR_EXIT;
@@ -267,7 +266,7 @@ extract_http_url(const char *url, struct request_s *request)
 static int
 extract_ssl_url(const char *url, struct request_s *request)
 {
-	request->host = safemalloc(strlen(url) + 1);
+	request->host = (char*)safemalloc(strlen(url) + 1);
 	if (!request->host)
 		return -1;
 
@@ -318,7 +317,8 @@ void
 upstream_add(const char *host, int port, const char *domain)
 {
 	char *ptr;
-	struct upstream *up = safemalloc(sizeof (struct upstream));
+	struct upstream *up = 
+		(struct upstream*)safemalloc(sizeof (struct upstream));
 
 	if (!up) {
 		log_message(LOG_ERR, "Unable to allocate memory in upstream_add()");
@@ -404,7 +404,7 @@ upstream_add(const char *host, int port, const char *domain)
 
 	return;
 
-      upstream_cleanup:
+upstream_cleanup:
 	safefree(up->host);
 	safefree(up->domain);
 	safefree(up);
@@ -465,6 +465,68 @@ upstream_get(char *host)
 }
 #endif
 
+#ifdef REVERSE_SUPPORT
+/*
+ * Add entry to the reversepath list
+ */
+void
+reversepath_add(const char *path, const char *url)
+{
+	struct reversepath *reverse;
+
+	if (url == NULL) {
+		log_message(LOG_WARNING, "Illegal reverse proxy rule: missing url");
+		return;
+	}
+
+	if (!strstr(url, "://")) {
+		log_message(LOG_WARNING,
+		            "Skipping reverse proxy rule: '%s' is not a valid url", url);
+		return;
+	}
+
+	if (path && *path != '/') {
+		log_message(LOG_WARNING,
+		            "Skipping reverse proxy rule: path '%s' doesn't start with a /", path);
+		return;
+	}
+
+	if (!(reverse = safemalloc(sizeof (struct reversepath)))) {
+		log_message(LOG_ERR, "Unable to allocate memory in reversepath_add()");
+		return;
+	}
+
+	if (!path) reverse->path = safestrdup("/");
+	else reverse->path = safestrdup(path);
+
+	reverse->url = safestrdup(url);
+
+	reverse->next = config.reversepath_list;
+	config.reversepath_list = reverse;
+
+	log_message(LOG_INFO,
+	            "Added reverse proxy rule: %s -> %s", reverse->path, reverse->url);
+}
+
+/*
+ * Check if a request url is in the reversepath list
+ */
+static struct reversepath *
+reversepath_get(char *url)
+{
+	struct reversepath *reverse = config.reversepath_list;
+
+	while (reverse) {
+		if (strstr(url, reverse->path) == url)
+			return reverse;
+
+		reverse = reverse->next;
+	}
+
+	return NULL;
+}
+#endif
+
 /*
  * Create a connection for HTTP connections.
  */
@@ -488,7 +550,7 @@ establish_http_connection(struct conn_s *connptr, struct request_s *request)
 }
 
 /*
- * These two defines are for the SSL tunneling.
+ * These two defines are for the SSL tunnelling.
  */
 #define SSL_CONNECTION_RESPONSE "HTTP/1.0 200 Connection established"
 #define PROXY_AGENT "Proxy-agent: " PACKAGE "/" VERSION
@@ -517,20 +579,27 @@ process_request(struct conn_s *connptr, hashmap_t hashofheaders)
 	char *url;
 	struct request_s *request;
 
+#ifdef REVERSE_SUPPORT
+	char *rewrite_url = NULL;
+	char *cookie = NULL;
+	char *cookieval;
+	struct reversepath *reverse;
+#endif
+
 	int ret;
 
 	size_t request_len;
 
 	/* NULL out all the fields so frees don't cause segfaults. */
-	request = safecalloc(1, sizeof(struct request_s));
+	request = (struct request_s*)safecalloc(1, sizeof(struct request_s));
 	if (!request)
 		return NULL;
 
 	request_len = strlen(connptr->request_line) + 1;
 
-	request->method = safemalloc(request_len);
-	url = safemalloc(request_len);
-	request->protocol = safemalloc(request_len);
+	request->method = (char*)safemalloc(request_len);
+	url = (char*)safemalloc(request_len);
+	request->protocol = (char*)safemalloc(request_len);
 
 	if (!request->method || !url || !request->protocol) {
 		safefree(url);
@@ -577,6 +646,66 @@ process_request(struct conn_s *connptr, hashmap_t hashofheaders)
 		return NULL;
 	}
 
+#ifdef REVERSE_SUPPORT
+	/*
+	 * Reverse proxy URL rewriting.
+	 */
+	if (config.reversepath_list != NULL) {
+		/* Reverse requests always start with a slash */
+		if (*url == '/') {
+			/* First try locating the reverse mapping by request url */
+			reverse = reversepath_get(url);
+			if (reverse) {
+				rewrite_url = safemalloc(strlen(url) +
+				              strlen(reverse->url) + 1);
+				strcpy(rewrite_url, reverse->url);
+				strcat(rewrite_url, url + strlen(reverse->path));
+			} else if (config.reversemagic &&
+			         hashmap_entry_by_key(hashofheaders, "cookie",
+			                              (void **)&cookie) > 0) {
+
+				/* No match - try the magical tracking cookie next */
+				if ((cookieval = strstr(cookie, REVERSE_COOKIE "=")) &&
+				    (reverse = reversepath_get(cookieval +
+				                               strlen(REVERSE_COOKIE) + 1))) {
+
+					rewrite_url = safemalloc(strlen(url) +
+					                         strlen(reverse->url) + 1);
+					strcpy(rewrite_url, reverse->url);
+					strcat(rewrite_url, url + 1);
+
+					log_message(LOG_INFO,
+					            "Magical tracking cookie says: %s",
+					            reverse->path);
+				}
+			}
+		}
+
+		/* Forward proxy support off and no reverse path match found */
+		if (config.reverseonly && !rewrite_url) {
+			log_message(LOG_ERR, "Bad request");
+			indicate_http_error(connptr, 400, "Bad Request",
+					    "detail", "Request has an invalid URL",
+					    "url", url,
+					    NULL);
+
+			safefree(url);
+			free_request_struct(request);
+
+			return NULL;
+		}
+
+		log_message(LOG_CONN, "Rewriting URL: %s -> %s",
+			    url, rewrite_url);
+
+		safefree(url);
+		url = rewrite_url;
+
+		/* Store reverse path so that the magical tracking cookie can be set */
+		if (config.reversemagic) connptr->reversepath = safestrdup(reverse->path);
+	}
+#endif
+
 	if (strncasecmp(url, "http://", 7) == 0
 	    || (UPSTREAM_CONFIGURED() && strncasecmp(url, "ftp://", 6) == 0)) {
 		char *skipped_type = strstr(url, "//") + 2;
@@ -726,7 +855,7 @@ process_request(struct conn_s *connptr, hashmap_t hashofheaders)
 					    request->host);
 
 			indicate_http_error(connptr, 403, "Filtered",
-					    "detail", "The request you made has been filted",
+					    "detail", "The request you made has been filtered",
 					    "url", url,
 					    NULL);
 
@@ -775,7 +904,7 @@ pull_client_data(struct conn_s *connptr, long int length)
 	char *buffer;
 	ssize_t len;
 
-	buffer = safemalloc(min(MAXBUFFSIZE, length));
+	buffer = (char*)safemalloc(min(MAXBUFFSIZE, length));
 	if (!buffer)
 		return -1;
 
@@ -1057,6 +1186,7 @@ process_client_headers(struct conn_s *connptr, hashmap_t hashofheaders)
 		"proxy-connection",
 		"te",
 		"trailers",
+		"transfer-encoding",
 		"upgrade"
 	};
 	int i;
@@ -1071,7 +1201,7 @@ process_client_headers(struct conn_s *connptr, hashmap_t hashofheaders)
 	 * proxy is in use.)
 	 */
 	if (connptr->server_fd == -1 || connptr->show_stats
-	    || (connptr->connect_method && (connptr->upstream_proxy == NULL))) {
+	    || (connptr->connect_method && !UPSTREAM_CONFIGURED())) {
 		log_message(LOG_INFO, "Not sending client headers to remote machine");
 		return 0;
 	}
@@ -1165,6 +1295,7 @@ process_server_headers(struct conn_s *connptr)
 		"proxy-authenticate",
 		"proxy-authorization",
 		"proxy-connection",
+		"transfer-encoding",
 	};
 
 	char *response_line;
@@ -1176,6 +1307,10 @@ process_server_headers(struct conn_s *connptr)
 	int i;
 	int ret;
 
+#ifdef REVERSE_SUPPORT
+	struct reversepath *reverse = config.reversepath_list;
+#endif
+
 	/* FIXME: Remember to handle a "simple_req" type */
 
 	/* Get the response line from the remote server. */
@@ -1249,6 +1384,41 @@ process_server_headers(struct conn_s *connptr)
 	if (ret < 0)
 		goto ERROR_EXIT;
 
+#ifdef REVERSE_SUPPORT
+	/* Write tracking cookie for the magical reverse proxy path hack */
+	if (config.reversemagic && connptr->reversepath) {
+		ret = write_message(connptr->client_fd,
+				    "Set-Cookie: " REVERSE_COOKIE "=%s; path=/\r\n",
+				    connptr->reversepath);
+		if (ret < 0) goto ERROR_EXIT;
+	}
+
+	/* Rewrite the HTTP redirect if needed */
+	if (config.reversebaseurl &&
+	    hashmap_entry_by_key(hashofheaders, "location", (void **)&header) > 0) {
+
+		/* Look for a matching entry in the reversepath list */
+		while (reverse) {
+			if (strncasecmp(header,
+			                reverse->url,
+			                (len = strlen(reverse->url))) == 0) break;
+			reverse = reverse->next;
+		}
+
+		if (reverse) {
+			ret = write_message(connptr->client_fd, "Location: %s%s%s\r\n",
+			                    config.reversebaseurl, (reverse->path + 1),
+			                    (header + len));
+			if (ret < 0) goto ERROR_EXIT;
+
+			log_message(LOG_INFO,
+			            "Rewriting HTTP redirect: %s -> %s%s%s", header,
+			            config.reversebaseurl, (reverse->path + 1), (header + len));
+			hashmap_remove(hashofheaders, "location");
+		}
+	}
+#endif
+
 	/*
 	 * All right, output all the remaining headers to the client.
 	 */
@@ -1408,7 +1578,7 @@ connect_to_upstream(struct conn_s *connptr, struct request_s *request)
 	char *combined_string;
 	int len;
 
-	struct upstream *cur_upstream = connptr->upstream_proxy;
+	struct upstream *cur_upstream = upstream_get(request->host);
 	if(!cur_upstream) {
 		log_message(LOG_WARNING,
 			    "No upstream proxy defined for %s.",
@@ -1441,7 +1611,7 @@ connect_to_upstream(struct conn_s *connptr, struct request_s *request)
 	if (connptr->connect_method) {
 		len = strlen(request->host) + 7;
 
-		combined_string = safemalloc(len);
+		combined_string = (char*)safemalloc(len);
 		if (!combined_string) {
 			return -1;
 		}
@@ -1450,7 +1620,7 @@ connect_to_upstream(struct conn_s *connptr, struct request_s *request)
 			 request->port);
 	} else {
 		len = strlen(request->host) + strlen(request->path) + 14;
-		combined_string = safemalloc(len);
+		combined_string = (char*)safemalloc(len);
 		if (!combined_string) {
 			return -1;
 		}
@@ -1552,8 +1722,7 @@ handle_connection(int fd)
 		goto send_error;
 	}
 
-	connptr->upstream_proxy = UPSTREAM_HOST(request->host);
-	if (connptr->upstream_proxy != NULL) {
+	if (UPSTREAM_CONFIGURED() && (UPSTREAM_HOST(request->host) != NULL)) {
 		if (connect_to_upstream(connptr, request) < 0) {
 			goto send_error;
 		}
@@ -1598,7 +1767,7 @@ handle_connection(int fd)
 		return;
 	}
 
-	if (!connptr->connect_method || (connptr->upstream_proxy != NULL)) {
+	if (!connptr->connect_method || UPSTREAM_CONFIGURED()) {
 		if (process_server_headers(connptr) < 0) {
 			if (connptr->error_variables)
 				send_http_error_message(connptr);
