@@ -1,4 +1,4 @@
-/* $Id: stats.c,v 1.13 2003-03-13 21:31:03 rjkaes Exp $
+/* $Id$
  *
  * This module handles the statistics for tinyproxy. There are only two
  * public API functions. The reason for the functions, rather than just a
@@ -45,7 +45,7 @@ static struct stat_s *stats;
 void
 init_stats(void)
 {
-	stats = malloc_shared_memory(sizeof(struct stat_s));
+	stats = (struct stat_s*)malloc_shared_memory(sizeof(struct stat_s));
 	if (stats == MAP_FAILED)
 		return;
 
@@ -81,7 +81,7 @@ showstats(struct conn_s *connptr)
 	snprintf(refused, sizeof(refused), "%lu", stats->num_refused);
 
 	if (!config.statpage || (!(statfile = fopen(config.statpage, "r")))) {
-		message_buffer = safemalloc(MAXBUFFSIZE);
+		message_buffer = (char*)safemalloc(MAXBUFFSIZE);
 		if (!message_buffer)
 			return -1;
 
