@@ -1,4 +1,4 @@
-/* $Id: tinyproxy.h,v 1.41.2.1 2004-08-06 16:56:55 rjkaes Exp $
+/* $Id$
  *
  * See 'tinyproxy.c' for a detailed description.
  *
@@ -20,15 +20,13 @@
 #define TINYPROXY_TINYPROXY_H
 
 #include "common.h"
+#include "hashmap.h"
 
 /* Global variables for the main controls of the program */
 #define MAXBUFFSIZE	((size_t)(1024 * 96))	/* Max size of buffer */
 #define MAX_IDLE_TIME 	(60 * 10)	/* 10 minutes of no activity */
 
-/*
- * Even if upstream support is not compiled into tinyproxy, this
- * structure still needs to be defined.
- */
+#ifdef UPSTREAM_SUPPORT
 struct upstream {
 	struct upstream *next;
 	char *domain; /* optional */
@@ -36,7 +34,21 @@ struct upstream {
 	int port;
 	in_addr_t ip, mask;
 };
+#endif
 
+#ifdef REVERSE_SUPPORT
+struct reversepath {
+	struct reversepath *next;
+	char *path;
+	char *url;
+};
+
+#define REVERSE_COOKIE "yummy_magical_cookie"
+#endif
+
+/*
+ * Hold all the configuration time information.
+ */
 struct config_s {
 	char *logf_name;
 	char *config_file;
@@ -56,6 +68,12 @@ struct config_s {
 #ifdef XTINYPROXY_ENABLE
 	char *my_domain;
 #endif
+#ifdef REVERSE_SUPPORT
+	struct reversepath *reversepath_list;
+	unsigned int reverseonly; /* boolean */
+	unsigned int reversemagic; /* boolean */
+	char *reversebaseurl;
+#endif
 #ifdef UPSTREAM_SUPPORT
 	struct upstream *upstream_list;
 #endif				/* UPSTREAM_SUPPORT */
@@ -68,16 +86,11 @@ struct config_s {
 	 */
 	char* via_proxy_name;
 
-	/* 
-	 * Error page support.  This is an array of pointers to structures
-	 * which describe the error page path, and what HTTP error it handles.
-	 * an example would be { "/usr/local/etc/tinyproxy/404.html", 404 }
-	 * Ending of array is noted with NULL, 0.
+        /* 
+	 * Error page support.  Map error numbers to file paths.
 	 */
-	struct error_pages_s {
-		char *errorpage_path;
-		unsigned int errorpage_errnum;
-	} **errorpages;
+	hashmap_t errorpages;
+
 	/* 
 	 * Error page to be displayed if appropriate page cannot be located
 	 * in the errorpages structure.
