@@ -1,4 +1,4 @@
-/* $Id: vector.c,v 1.9 2003/05/30 16:21:48 rjkaes Exp $
+/* $Id$
  *
  * A vector implementation.  The vector can be of an arbitrary length, and
  * the data for each entry is an lump of data (the size is stored in the
@@ -58,7 +58,7 @@ vector_create(void)
 {
 	vector_t vector;
 
-	vector = safemalloc(sizeof(struct vector_s));
+	vector = (vector_t)safemalloc(sizeof(struct vector_s));
 	if (!vector)
 		return NULL;
 
@@ -118,7 +118,7 @@ vector_insert(vector_t vector, void *data, ssize_t len, int pos)
 	    (pos != INSERT_PREPEND && pos != INSERT_APPEND))
 		return -EINVAL;
 
-	entry = safemalloc(sizeof(struct vectorentry_s));
+	entry = (struct vectorentry_s*)safemalloc(sizeof(struct vectorentry_s));
 	if (!entry)
 		return -ENOMEM;
 
